/*
 * Author: Richard Lord
 * Copyright (c) Richard Lord 2010
 * http://www.richardlord.net
 * 
 * Licence Agreement
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package net.richardlord.smartcomponents.common.containers
{
	import flash.display.DisplayObject;
	import flash.events.Event;
	
	import mx.containers.Canvas;
	import mx.containers.ViewStack;
	import mx.core.UIComponent;
	
	public class ToggleVisibleCanvas extends Canvas
	{
		public function ToggleVisibleCanvas()
		{
			super();
			visible = false;
			includeInLayout = false;
		}
		
		protected function getContainer() : UIComponent
		{
			var container : DisplayObject = this.parent;
			while ( !( container is UIComponent ) )
			{
				container = container.parent;
			}
			return container as UIComponent;
		}
		
		protected function toggleMe( event:Event = null ) : void
		{
			if ( visible )
			{
				hideMe();
			}
			else
			{
				showMe();
			}
		}
		
		protected function showMe( event:Event = null ) : void
		{
			var container : UIComponent = getContainer();
			if ( container is ViewStack )
			{
				ViewStack( container ).selectedChild = this;
			}
			else if ( container is ZeroViewStack )
			{
				ZeroViewStack( container ).selectedChild = this;
			}
			else
			{
				visible = true;
				includeInLayout = true;
			}
		}
		
		protected function hideMe( event:Event = null ) : void
		{
			var container : UIComponent = getContainer();
			if ( container is ViewStack )
			{
				if ( ViewStack( container ).selectedChild == this )
				{
					ViewStack( container ).selectedIndex = 0;
				}
			}
			else if ( container is ZeroViewStack )
			{
				if ( ZeroViewStack( container ).selectedChild == this )
				{
					ZeroViewStack( container ).selectedIndex = -1;
				}
			}
			else
			{
				visible = false;
				includeInLayout = false;
			}
		}
	}
}