/*
 * Author: Richard Lord
 * Copyright (c) Richard Lord 2010
 * http://www.richardlord.net
 * 
 * Licence Agreement
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package net.richardlord.smartcomponents.twitter.managers
{
	import flash.events.Event;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.net.navigateToURL;
	
	import mx.core.Window;
	
	import net.richardlord.smartcomponents.twitter.context.EventBus;
	import net.richardlord.smartcomponents.twitter.data.User;
	import net.richardlord.smartcomponents.twitter.events.AuthenticateEvent;
	
	import org.flaircode.oauth.IOAuth;
	import org.flaircode.oauth.OAuthLoader;
	import org.iotashan.oauth.OAuthToken;
	import org.iotashan.utils.OAuthUtil;
	
	public class AuthenticateManager
	{
		private const requestTokenUrl : String = "http://twitter.com/oauth/request_token";
		private const accessTokenUrl : String = "http://twitter.com/oauth/access_token";
		private const authorizeUrl : String = "http://twitter.com/oauth/authorize";
		
		private var requestToken : OAuthToken;
		private var accessToken : OAuthToken;
		
		[Inject]
		public var user : User;
		
		[Inject]
		public var oauth : IOAuth;
		
		[Inject]
		public var homeManager : HomeManager;
		
		[Inject]
		public var eventBus : EventBus;
		
		public var authenticateInAir : Boolean = false;
		
		public var postAuthenticateEvent : Event;
		
		public function authenticate() : void
		{
			var loader : URLLoader = oauth.getRequestToken( requestTokenUrl );
			loader.addEventListener( Event.COMPLETE, requestTokenHandler );
		}
		
		private function requestTokenHandler( e : Event ) : void
		{
			requestToken = OAuthUtil.getTokenFromResponse( e.currentTarget.data as String );
			var request : URLRequest = oauth.getAuthorizeRequest( authorizeUrl, requestToken.key );
			if ( authenticateInAir )
			{
				var loader : OAuthLoader = new OAuthLoader();
				loader.load( request );
				loader.percentWidth = 100;
				loader.percentHeight = 100;
				var w : Window = new Window();
				w.width = 800;
				w.height = 400;
				w.title = request.url;
				w.addChild( loader );
				w.open();
			}
			else
			{
				navigateToURL( request, "_blank" );
			}
		}
		
		public function getAccessToken( pin : String ) : void
		{
			var loader : URLLoader = oauth.getAccessToken( accessTokenUrl, requestToken, { oauth_verifier: pin } );
			loader.addEventListener( Event.COMPLETE, accessTokenHandler );
		}
		
		private function accessTokenHandler( e : Event ) : void
		{
			accessToken = OAuthUtil.getTokenFromResponse( e.currentTarget.data as String );
			user.token = accessToken;
			homeManager.load( 20 );
			if ( postAuthenticateEvent )
			{
				eventBus.dispatchEvent( postAuthenticateEvent );
				postAuthenticateEvent = null;
			}
			eventBus.dispatchEvent( new AuthenticateEvent( AuthenticateEvent.HIDE_AUTHENTICATE ) );
		}
	}
}