/*
 * Author: Richard Lord
 * Copyright (c) Richard Lord 2010
 * http://www.richardlord.net
 * 
 * Licence Agreement
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package net.richardlord.smartcomponents.twitter.services.parsers
{
	import flash.net.URLVariables;
	
	import net.richardlord.smartcomponents.twitter.data.Tweet;
	import net.richardlord.smartcomponents.twitter.data.TweetCollection;
	
	public class TimelineAtomParser implements Parser
	{
		private static const at : Namespace = new Namespace( "http://www.w3.org/2005/Atom" );
		
		private static const geo : Namespace = new Namespace( "http://www.georss.org/georss" );
		
		public function parseFeed( feed : XML ) : TweetCollection
		{
			var tweetCollection : TweetCollection = new TweetCollection();
			
			tweetCollection.id = feed.at::id.text();
			tweetCollection.title = feed.at::title.text();
			tweetCollection.subtitle = feed.at::subtitle.text();
			tweetCollection.webUrl = parseUrl( feed.at::link.( @type == "text/html" && @rel == "alternate" ).@href.toString() );
			tweetCollection.selfUrl = parseUrl( feed.at::link.( @type == "application/atom+xml" && @rel == "self" ).@href.toString() );
			tweetCollection.updated = parseDate( feed.at::updated.text() );
			tweetCollection.language = feed.@lang.toString();
			tweetCollection.parser = this;
			
			var tweets : Array = new Array();
			for each ( var entry : XML in feed.at::entry )
			{
				tweets.push( parseTweet( entry ) );
			}
			tweetCollection.tweets = tweets;
			
			var latestId : Number;
			var oldestId : Number;
			if ( tweets.length != 0 )
			{
				latestId = Tweet( tweets[ 0 ] ).id;
				oldestId = Tweet( tweets[ tweets.length - 1 ] ).id;
			}
			
			var url : String = tweetCollection.selfUrl;
			var variables : URLVariables = new URLVariables();
			var questionLoc : int = url.indexOf( "?" );
			if ( questionLoc != -1 )
			{
				variables.decode( url.substring( questionLoc + 1 ) );
				url = url.substring( 0, questionLoc );
			}
			delete variables.page;
			
			if ( oldestId )
			{
				variables.max_id = oldestId - 1;
				tweetCollection.nextUrl = url + "?" + variables.toString();
				delete variables.max_id;
			}
			
			if ( variables.count )
			{
				tweetCollection.itemsPerPage = parseInt( variables.count );
				delete variables.count;
			}
			else
			{
				tweetCollection.itemsPerPage = 20;
			}
			
			if ( latestId )
			{
				variables.since_id = latestId;
				tweetCollection.refreshUrl = url + "?" + variables.toString();
			}
			return tweetCollection;
		}
		
		private function parseTweet( entry : XML ) : Tweet
		{
			var tweet : Tweet = new Tweet();
			var id : String = entry.at::id.text();
			var matches : Array = id.match( /\d*$/ );
			tweet.id = parseFloat( matches[ 0 ] );
			tweet.created = parseDate( entry.at::published.text() );
			tweet.link = entry.at::link.( @type == "text/html" && @rel == "alternate" ).@href.toString()
			tweet.text = entry.at::title.text();
			tweet.html = entry.at::content.( @type = "html" ).text();
			tweet.image = entry.at::link.( @rel == "image" ).@href.toString();
			tweet.source = "";
			tweet.authorName = entry.at::author.at::name.text();
			tweet.authorUri = entry.at::author.at::uri.text();
			return tweet;
		}
		
		private function parseDate( time : String ) : Date
		{
			var year : int = parseInt( time.substr( 0, 4 ) );
			var month : int = parseInt( time.substr( 5, 2 ) ) - 1;
			var day : int = parseInt( time.substr( 8, 2 ) );
			var hour : int = parseInt( time.substr( 11, 2 ) );
			var minute : int = parseInt( time.substr( 14, 2 ) );
			var second : int = parseInt( time.substr( 17, 2 ) );
			if ( time.substr( 19, 1 ) == "-" )
			{
				hour += parseInt( time.substr( 20, 2 ) );
				minute += parseInt( time.substr( 23, 2 ) );
			}
			else if ( time.substr( 19, 1 ) == "+" )
			{
				hour -= parseInt( time.substr( 20, 2 ) );
				minute -= parseInt( time.substr( 23, 2 ) );
			}
			return new Date( Date.UTC( year, month, day, hour, minute, second ) );
		}
		
		private function parseUrl( url : String ) : String
		{
			var split : Array = url.split( "?" );
			if ( split.length == 1 )
			{
				return url;
			}
			var base : String = split[ 0 ];
			var vars : URLVariables = new URLVariables( split[ 1 ] );
			var vars2 : URLVariables = new URLVariables( split[ 1 ] );
			for ( var name : String in vars2 )
			{
				if ( name.substr( 0, 6 ) == "oauth_" )
				{
					delete( vars[ name ] );
				}
			}
			return base + "?" + vars.toString();
		}
	}
}